let wc_form_data = [];
let wc_site_url, wc_api_loc, wc_nonce, wc_template_loc, wc_user_data, wc_page_class;
let page_title, page_container;
const reactivate_form_time = 500;

const replace_string = '/data/attributes/';
const localLanguage = navigator.language ?? 'en-US';
let modal_name;
let checkup_details_id = 0, checkup_time_id, AuthTokenWebsiteCheckups;

let btn_popup_manage_notification;

let btn_popup_manage_checkup;

let uid_counter = 0;
let cached_endpoints = ['countries', 'countries?filter[popular]=1', 'languages', 'languages?filter[popular]=1',
    'subproducts', 'fielddescriptions'];
//let tmp_cached_endpoints = ['users', 'subscriptions', 'registrations', 'notifications', 'invoices'];
async function fetch_template(file_name) {
    try {
        const file_path = wc_template_loc + file_name;
        const response = await fetch(file_path);

        if (!response.ok) {
            console.error('Error:', 'template fetch error');
        }
        return await response.text();
    } catch (error) {
        console.error('Error:', error);
        return false;
    }
}
function get_webcheckups_storage_data(key)
{
    if (localStorage) {
        const time = Math.ceil((new Date()).getTime()/1000);
        const data_string = localStorage.getItem('webcheckups:' + key);
        const data_obj = data_string ? JSON.parse(data_string) : null;
        const time_diff = data_obj?.saved ? time - data_obj?.saved : 9999999;

        if (data_obj?.data && time_diff < 24 * 60 * 60) {
            return data_obj?.data ?? null;
        }
    }
    return null;
}

function set_webcheckups_storage_data(key, data)
{
    if (localStorage) {
        const time = Math.ceil((new Date()).getTime()/1000);
        localStorage.setItem('webcheckups:' + key, JSON.stringify({'saved': time, 'data': data}));
    }
}

async function manage_api_data(api_end_point, api_method, form_data = null)
{
    if (api_method == 'get' && cached_endpoints.includes(api_end_point)) {
        const ret = get_webcheckups_storage_data(api_end_point);
        if (ret) {
            return ret;
        }
    }

    show_loader();
    let url = wc_api_loc + api_end_point;

    let headers = {
        'Content-Type': 'application/vnd.api+json', 'Accept': 'application/vnd.api+json',
    };

    if (wc_user_data['token']) {
        headers['Authorization'] = 'Bearer ' + wc_user_data['token'];
    }

    let options = {
        method: api_method.toUpperCase(), headers: headers,
    };

    let input_data = {
        data: {
            type: api_end_point,
        },
    };

    if (form_data !== null) {
        if (api_method !== 'get' && api_method !== 'post') {
            if (form_data['item_id']) {
                url += '/' + form_data['item_id'];
                input_data['data']['id'] = form_data['item_id'];
                delete form_data['item_id'];
            }
        }

        if (api_method === 'post') {
            if (form_data['bulk_select']) {
                url = wc_api_loc + 'bulk/' + api_end_point + '/' + form_data['bulk_type'];
                delete form_data['bulk_select'];
                delete form_data['bulk_type'];
            }
        }

        input_data['data']['attributes'] = form_data;
        options.body = JSON.stringify(input_data);
    }

    try {
        const response = await fetch(url, options);
        const responseData = await response.json();
        if (!response.ok) {
            console.error('Error:', responseData);
        }
        if (api_method == 'get' && cached_endpoints.includes(api_end_point)) {
            set_webcheckups_storage_data(api_end_point, responseData);
        }
        return responseData;
    } catch (error) {
        // console.error('Error:', error);
        return false;
    }
}

/* replace manage_local_data for some forms */
async function prepare_form_fields(form) {
    const form_data = await manage_api_data('fielddescriptions', 'get');

    const form_fields = form_data.data ?
        form_data.data.filter(e => e.attributes.form==form).map(item => item.attributes) : [];

    const form_fields_mapped = form_fields ? form_fields.map(item => ({
        'field_item': document.getElementById(item.field),
        'field_name': item.field,
        'field_type': item.type,
        'field_advanced': item.advanced === 1 ? 1 : null,
        'field_required': (item.required === 1),
        'values_description': item.valuesDescription,
        'disable_for': item.disableFor,
        'detail': item.required == 1 && item?.label ? item.label + ' is required' : null
    })) : [];

    return form_fields_mapped.length ? form_fields_mapped : [];
}

function manage_local_data(post_data) {
    post_data['action'] = 'manage_admin_data';
    post_data['nonce'] = wc_nonce;

    fetch(ajaxurl, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
        },
        body: new URLSearchParams(post_data)
    }).then(response => response.json())
        .then(async result => {
            if (result['success'] && post_data['post_type'] === 'load_form') {
                const tmp_result = {};
                result['data'].forEach(form => {
                    form.fields.forEach(field => {
                        const form_name = field['form_name'];
                        if (!tmp_result[form_name]) {
                            tmp_result[form_name] = [];
                        }

                        const tmp_data = {
                            'field_item': document.getElementById(field['field_name']),
                            'field_name': field['field_name'],
                            'field_type': field['field_type'],
                            'field_advanced': field['field_advanced'],
                            'field_required': field['field_required'],
                            'values_description': field['values_description'],
                            'disable_for': field['disableFor'],
                            'detail': field['detail']
                        };

                        tmp_result[form_name].push(tmp_data);
                    });
                });
                wc_form_data = tmp_result;
            }
            if (result['success'] && post_data['post_type'] === 'user_register' && wc_page_class === 'toplevel_page_website_checkups_menu') {
                setTimeout(function () {
                    location.reload();
                }, 2000);

                if (Array.isArray(result['data']['msg'])) {
                    result['data']['msg'].forEach((single_msg, index) => {
                        show_toast(result['success'], single_msg, (index + 1));
                    });
                } else {
                    show_toast(result['success'], result['data']['msg'], 1);
                }
            } else {
                enable_all_fieldset();
            }
        })
        .catch(error => {
            console.error('Error:', error);
        });
}

function generate_unique_id() {
    const timestamp = Date.now();
    uid_counter = (uid_counter + 1) % 1000;
    return `id-${timestamp}-${uid_counter.toString().padStart(3, '0')}`;
}

function show_toast(toast_status, toast_msg, toast_delay = 1) {
    clear_all_toast();
    if (toast_msg !== undefined) {
        fetch_template('template_toast.html')
            .then(template_data => {
                const timestamp = Date.now().toString(36);
                const randomPart = Math.random().toString(36).substring(2, 8);
                const unique_id = timestamp + '-' + randomPart;

                let toast_color = 'danger';

                if (toast_status) {
                    toast_color = 'success';
                }

                if (toast_status === 'light') {
                    toast_color = 'Light';
                }

                template_data = template_data.replace('{{toast_id}}', unique_id);
                template_data = template_data.replace('{{toast_color}}', toast_color);
                template_data = template_data.replace('{{toast_msg}}', toast_msg);

                const existing_toasts = document.querySelectorAll('.toast .toast-body');
                for (let i = 0; i < existing_toasts.length; i++) {
                    if (existing_toasts[i].textContent.trim() === toast_msg.trim()) {
                        return;
                    }
                }
                document.getElementById('toast_holder').innerHTML += template_data;

                const toastElList = [].slice.call(document.querySelectorAll('.toast'));
                const toastList = toastElList.map(function (toastEl) {
                    const toast = new bootstrap.Toast(toastEl);
                    toastEl.addEventListener('hidden.bs.toast', () => {
                        toastEl.remove();
                    });

                    return toast;
                });
                toastList.forEach(toast => toast.show({delay: toast_delay * 1000}));

            }).catch(error => {
            console.error('Error:', error);
        });
    }
}

function clear_all_toast() {
    document.getElementById('toast_holder').innerHTML = '';
}

function clear_all_field(form_field) {
    if (!form_field || Object.keys(form_field).length == 0) {
        return false;
    }

    form_field.forEach(item => {
        const field = item['field_item'];
        if (field?.value) {
            field.value = '';
        }
        if (field?.checked) {
            field.checked = false;
        }
    });
}

function field_error_clear(fields) {
    fields.forEach(item => {
        const field = item['field_item'];
        const removeErrorClass = () => {
            field.classList.remove('is-invalid');
        };

        field.addEventListener('focus', removeErrorClass);
        field.addEventListener('change', removeErrorClass);
    });
}

function remove_all_error_class() {
    const error_input = document.querySelectorAll('.is-invalid');
    error_input.forEach(input => {
        input.classList.remove('is-invalid');
    });
}

function generate_multiple_toast(items) {
    if (items?.length > 0) {
        const ol = document.createElement('ol');
        ol.style.margin = '0px';

        items.forEach(item => {
            if (item['detail']) {
                const li = document.createElement('li');
                li.textContent = item['detail'];
                ol.appendChild(li);

                if (item['source'] != null) {
                    const tmp_item_name = item['source']['pointer'].replace(replace_string, '');
                    generate_field_error(Object.values(wc_form_data).flat().find(item => item['field_name'] === tmp_item_name)['field_item']);
                }
            }
        });
        if (ol.childElementCount > 0) {
            show_toast(false, ol.outerHTML, 1);
        }
    }
}

function field_error_check(fields) {
    let has_error = false;
    field_error_clear(fields);
    let tmp_required_field = [];

    fields.forEach(field => {
        const isFieldRequired = (field['field_required'] !== false);
        if ((field['field_item'].getAttribute('type') !== 'hidden' && isFieldRequired)) {
            if (field['field_item'].value.trim() === '' || (field['field_item'].checked === false && field['field_item'].getAttribute('type') === 'checkbox')) {
                generate_field_error(field['field_item']);
                has_error = true;
                tmp_required_field.push(field);
            }
        }
    });
    if (has_error) {
        generate_multiple_toast(tmp_required_field);
    }
    return has_error;
}

function generate_field_error(field) {
    field.classList.add('is-invalid');
}

function enable_all_fieldset() {
    const all_fieldset = document.querySelectorAll('body fieldset');
    all_fieldset.forEach(single_fieldset => {
        single_fieldset.disabled = false;
        if (single_fieldset.querySelector('.spinner-border')?.classList) {
            single_fieldset.querySelector('.spinner-border').classList.add('d-none');
        }
    });
}

function toggle_disable_frm(btn_element) {
    const parent_fieldset = btn_element.closest('fieldset');
    const spinner = btn_element.querySelector('.spinner-border');

    if (!parent_fieldset.disabled) {
        parent_fieldset.disabled = true;
        spinner.classList.remove('d-none');
    } else {
        parent_fieldset.disabled = false;
        spinner.classList.add('d-none');
    }
}

function clear_all_form() {
    const field_sets = document.querySelectorAll('fieldset');

    field_sets.forEach(fieldset => {
        const spinners = document.querySelectorAll('.spinner-border');
        fieldset.disabled = false;
        spinners.forEach(spinner => {
            spinner.classList.add('d-none');
        })
    });
}

function generate_dropdown_list(dropdown_data, default_txt, data_attr, select_id = null, short_data = true) {
    let optionsHtml = '';

    if (default_txt !== null) {
        optionsHtml += '<option value="">' + default_txt + '</option>';
    }

    const tmp_dropdown_data = short_data ? short_data_alphabetically(dropdown_data, data_attr) : dropdown_data;

    tmp_dropdown_data.forEach(item => {
        const value = item['id'] !== undefined ? item['id'] : item['value'];
        const text = item['attributes'] && item['attributes'][data_attr] ? item['attributes'][data_attr] : item['text'];

        const selectedAttribute = (value === select_id) ? ' selected' : '';
        optionsHtml += '<option value="' + value + '" ' + selectedAttribute + '>' + text + '</option>';
    });

    return optionsHtml;
}

function generate_subscription_lst(dropdown_data, selected_id = null) {
    let option_group_free = '<optgroup label="Free">';
    let option_group_yearly = '<optgroup label="Yearly abo website checkups">';

    dropdown_data.forEach(sub_product => {
        const attributes = sub_product['attributes'];
        const optionText = `${attributes['name']}, max. ${attributes['max']} check${attributes['max'] > 1 ? 's' : ''} each ` + format_time(parseInt(attributes['frequency']));
        const selected = (parseInt(sub_product['id']) === selected_id) ? ' selected' : '';
        const item_option = `<option value="${sub_product['id']}"${selected}>${optionText}</option>`;

        if (attributes.name === 'Free') {
            option_group_free += item_option;
        } else {
            option_group_yearly += item_option;
        }
    });

    option_group_free += '</optgroup>';
    option_group_yearly += '</optgroup>';

    return option_group_free + option_group_yearly;
}

function generate_subscription_options(dropdown_data, abo_id = 1, selected_id = null) {
    let abo_variants = '';
    const dData = dropdown_data.find(item => item.id == abo_id).attributes.checkAlternatives;

    dData.forEach(variant => {
        const optionText = variant.text;
        const selected = (parseInt(variant.max) === selected_id) ? ' selected' : '';
        const item_option = `<option value="${variant['max']}"${selected}>${optionText}</option>`;

        abo_variants += item_option;
    });

    return abo_variants;
}

function generate_grouped_dropdown_list(dropdownData, optionTextKeys = [], show_bracket = false, selectedIds = []) {
    const optionGroups = {};

    dropdownData.forEach(single_dropdown_data => {
        const attributes = single_dropdown_data['attributes'];
        const notificationType = attributes['notificationType'];

        let optionTextParts = optionTextKeys.map(key => attributes[key] || '').filter(part => part);

        if (show_bracket && optionTextParts.length > 1) {
            const lastValue = optionTextParts.pop(); // Extract last value
            optionTextParts.push(' (' + lastValue + ')');
        }

        const selected = selectedIds.includes(single_dropdown_data['id']) ? ' selected' : '';
        const itemOption = '<option value="' + single_dropdown_data['id'] + '" ' + selected + '>' + optionTextParts.join(' ') + '</option>';

        if (!optionGroups[notificationType]) {
            optionGroups[notificationType] = '<optgroup label="' + notificationType.charAt(0).toUpperCase() + notificationType.slice(1) + '">';
        }
        optionGroups[notificationType] += itemOption;
    });

    for (const type in optionGroups) {
        optionGroups[type] += '</optgroup>';
    }

    return Object.values(optionGroups).join('');
}

function format_time(value) {
    if (value < 60) {
        return value + ' minute' + (value > 1 ? 's' : '');
    } else if (value < 1440) {
        const hours = Math.floor(value / 60);
        return hours + ' hour' + (hours > 1 ? 's' : '');
    } else if (value < 10080) {
        const days = Math.floor(value / 1440);
        return days + ' day' + (days > 1 ? 's' : '');
    } else if (value < 43200) {
        const weeks = Math.floor(value / 10080);
        return weeks + ' week' + (weeks > 1 ? 's' : '');
    } else {
        const months = Math.floor(value / 43200);
        return months + ' month' + (months > 1 ? 's' : '');
    }
}
function format_num(number, fractionDigits= 3) {
    const options = fractionDigits == 0 ? {} : { minimumFractionDigits: fractionDigits };
    return new Intl.NumberFormat(navigator.language, options).format(
        number,
    );
}

function format_date(dateString) {
    if (!dateString) {
        return;
    }
    const dateTime = new Date(dateString);

    if (isNaN(dateTime.getTime())) {
        console.error('Error:', dateString);
        return null;
    }

    const day = String(dateTime.getDate()).padStart(2, '0');
    const monthShort = dateTime.toLocaleString(localLanguage ?? 'default', {month: 'short'});
    const year = dateTime.getFullYear();

    const hours = String(dateTime.getHours()).padStart(2, '0');
    const minutes = String(dateTime.getMinutes()).padStart(2, '0');

    const hasTime = dateString.trim().length > 10;

    if (hasTime) {
        return `${monthShort} ${day} ${year} ${hours}:${minutes}`;
    } else {
        return `${monthShort} ${day} ${year}`;
    }
}

function get_subscription_name(product_data, search_id) {
    const item = product_data['data'].find(item => parseInt(item.id) === search_id);
    return item ? item.attributes.name : null;
}

function html_string_to_array(htmlString) {
    const tempDiv = document.createElement('div');
    tempDiv.innerHTML = htmlString;
    return Array.from(tempDiv.querySelectorAll('option')).map(option => option.value);
}

function toggle_switch(switch_id, dropdown_id, data_one = null, data_two = null) {
    switch_id.onchange = function () {
        const tmp_dropdown_value = dropdown_id.value;
        if (switch_id.checked) {
            dropdown_id.innerHTML = data_two;

            if (!html_string_to_array(data_two).includes(tmp_dropdown_value)) {
                dropdown_id.selectedIndex = 0;
            } else {
                dropdown_id.value = tmp_dropdown_value;
            }

            switch_id.value = 1;
        } else {
            dropdown_id.innerHTML = data_one;

            if (!html_string_to_array(data_one).includes(tmp_dropdown_value)) {
                dropdown_id.selectedIndex = 0;
            } else {
                dropdown_id.value = tmp_dropdown_value;
            }

            switch_id.value = 0;
        }
    };
}

function short_data_alphabetically(data, sortType) {
    return data.sort((a, b) => {
        if (a.attributes[sortType] < b.attributes[sortType]) {
            return -1;
        }
        if (a.attributes[sortType] > b.attributes[sortType]) {
            return 1;
        }
        return 0;
    });
}

function createAnimatedRect(x, y, scale) {
    const rect = document.createElementNS('http://www.w3.org/2000/svg', 'rect');
    rect.setAttribute('fill', '#000000');
    rect.setAttribute('stroke', '#000000');
    rect.setAttribute('stroke-width', '11');
    rect.setAttribute('width', 30 * scale);
    rect.setAttribute('height', 30 * scale);
    rect.setAttribute('x', x);
    rect.setAttribute('y', y);

    const animate = document.createElementNS('http://www.w3.org/2000/svg', 'animate');
    animate.setAttribute('attributeName', 'opacity');
    animate.setAttribute('calcMode', 'spline');
    animate.setAttribute('dur', '1.7s');
    animate.setAttribute('values', '1;0;1;');
    animate.setAttribute('keySplines', '.5 0 .5 1;.5 0 .5 1');
    animate.setAttribute('repeatCount', 'indefinite');

    if (x === 25 * scale) {
        animate.setAttribute('begin', '-.4s');
    } else if (x === 85 * scale) {
        animate.setAttribute('begin', '-.2s');
    } else if (x === 145 * scale) {
        animate.setAttribute('begin', '0s');
    }

    rect.appendChild(animate);
    return rect;
}

function show_loader(transparent_bg = true, loader_id = 'page_loader', loader_location = '.loader_block') {
    if (document.getElementById(loader_id)) {
        return;
    }
    const desiredHeight = 100;
    const originalWidth = 200;
    const originalHeight = 200;
    const scaleFactor = desiredHeight / originalHeight;
    const desiredWidth = originalWidth * scaleFactor;

    const loaderDiv = document.createElement('div');
    loaderDiv.id = loader_id;
    loaderDiv.className = 'loader';
    if (transparent_bg) {
        loaderDiv.classList.add('transparent_bg');
    }

    const svgLoader = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
    svgLoader.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
    svgLoader.setAttribute('viewBox', '0 0 200 200');
    svgLoader.setAttribute('width', desiredWidth);
    svgLoader.setAttribute('height', desiredHeight);

    const rect1 = createAnimatedRect(25 * scaleFactor, 85 * scaleFactor, scaleFactor);
    const rect2 = createAnimatedRect(85 * scaleFactor, 85 * scaleFactor, scaleFactor);
    const rect3 = createAnimatedRect(145 * scaleFactor, 85 * scaleFactor, scaleFactor);

    svgLoader.appendChild(rect1);
    svgLoader.appendChild(rect2);
    svgLoader.appendChild(rect3);

    loaderDiv.appendChild(svgLoader);

    const wrapElement = document.querySelector(loader_location);
    if (wrapElement !== null) {
        wrapElement.insertBefore(loaderDiv, wrapElement.firstChild);

        document.querySelector('body').classList.add('no_scroll');
    }
}

function close_loader(loader_id = 'page_loader') {
    const loader = document.getElementById(loader_id);
    if (loader) {
        loader.remove();
        document.querySelector('body').classList.remove('no_scroll');
    }
}

function search_tbl(tbl_id, search_value, col_indices) {
    const rows = tbl_id.getElementsByTagName('tbody')[0].getElementsByTagName('tr');

    const lowerCaseSearchValue = search_value.toLowerCase();

    for (let i = 0; i < rows.length; i++) {
        let rowContainsSearchValue = false;

        for (let j = 0; j < col_indices.length; j++) {
            const col_index = col_indices[j];
            const cell = rows[i].getElementsByTagName('td')[col_index];

            if (cell) {
                const cellText = (cell.textContent || cell.innerText).toLowerCase();

                if (cellText.indexOf(lowerCaseSearchValue) > -1) {
                    rowContainsSearchValue = true;
                    break;
                }
            }
        }

        if (rowContainsSearchValue) {
            rows[i].style.display = '';
        } else {
            rows[i].style.display = 'none';
        }
    }
}

function toggle_all_check_box(main_check_box, children_chk_box) {
    main_check_box.addEventListener('change', function () {
        children_chk_box.forEach(checkbox => {
            checkbox.checked = main_check_box.checked;
        });
    });

    children_chk_box.forEach(checkbox => {
        checkbox.addEventListener('change', function () {
            if (!checkbox.checked) {
                main_check_box.checked = false;
            }
        });
    });
}

function confirm_window(confirm_msg) {
    return confirm(confirm_msg);
}

function reset_form_field() {
    const modal_element = document.getElementById('modal_' + modal_name);
    const elements = modal_element.querySelectorAll('input, select, button, textarea');
    elements.forEach(element => {
        element.value = '';
    });
}

function close_modal(modal_id) {
    bootstrap.Modal.getInstance(document.getElementById(modal_id)).hide();
}

function get_most_recent_time(checkups = [], mode) {
    let mostRecentTime = null;

    checkups.forEach(checkup => {
        if (mode === 'fast') {
            if (mostRecentTime === null ||
                checkup.time < mostRecentTime.time ||
                (checkup.time === mostRecentTime.time && new Date(checkup.performed) > new Date(mostRecentTime.performed))) {
                mostRecentTime = checkup;
            }
        } else if (mode === 'slow') {
            if (mostRecentTime === null ||
                checkup.time > mostRecentTime.time ||
                (checkup.time === mostRecentTime.time && new Date(checkup.performed) > new Date(mostRecentTime.performed))) {
                mostRecentTime = checkup;
            }
        }
    });

    return mostRecentTime;
}

function subscription_info(product_list, current_item) {
    let msg_string;
    const element_subscription_info = document.getElementById('element_subscription_info');

    const selected_product_data = product_list['data'].find(item => parseInt(item['id']) === parseInt(current_item));

    if (parseInt(selected_product_data['id']) !== 1) {
        msg_string = 'This subscription will cost <strong>€ ' + selected_product_data['attributes']['price'] + '</strong> annually, or around <strong>€ ' + parseInt(selected_product_data['attributes']['price']) / 12 + '</strong> per month. Plus value added tax. Billing takes place once a year.';
    } else {
        msg_string = 'This subscription is and remains permanently <strong>free</strong> for you.';
    }

    element_subscription_info.innerHTML = msg_string;
}

function generate_filter_dropdown(dropdown_items, dropdown_element, data_info = null) {
    dropdown_items.forEach(item => {
        const option = document.createElement('option');
        option.value = make_lowercase_txt(item);
        option.textContent = item;
        if (data_info && data_info[make_lowercase_txt(item)]) {
            option.setAttribute('data-tooltip', data_info[make_lowercase_txt(item)]);
        }
        dropdown_element.appendChild(option);
    });
}

function make_lowercase_txt(searchString) {
    return searchString
        .split(' ')
        .map((word, index) =>
            index === 0
                ? word.toLowerCase()
                : word.charAt(0).toUpperCase() + word.slice(1).toLowerCase()
        )
        .join('');
}

function add_tooltip_txt(input_element, tooltip_data) {
    const parentElement = input_element.parentElement.parentElement;
    const nested_tooltip = parentElement.querySelector('.info_tooltip');
    nested_tooltip.setAttribute('data-info', tooltip_data);
}

function filter_dropdown_items(data, dropdown_element, filter_key) {
    const notificationTypes = new Set(
        data.map(item => item.attributes[filter_key])
    );

    Array.from(dropdown_element.options).forEach(option => {
        if (option.value && !notificationTypes.has(option.value)) {
            option.style.display = 'none';
        } else {
            option.style.display = '';
        }
    });
}

function generate_advance_block(block_id, block_btn_id, modal_id, form_element) {

    const advanced_block_btn = document.getElementById(block_btn_id);
    let advanced_block = document.getElementById(block_id);

    advanced_block_btn.classList.remove('d-none');

    if (advanced_block === null) {
        advanced_block = document.createElement('div');
        advanced_block.id = block_id;
        advanced_block.classList.add('row');
        document.querySelector(modal_id + ' .modal-body').appendChild(advanced_block);

        advanced_block.classList.add('d-none');

        advanced_block_btn.addEventListener('click', function () {
            advanced_block.classList.toggle('d-none');
        });
    }

    let parent = form_element.parentElement;

    while (parent !== null) {
        if (parent.classList.contains('input-group')) {
            const transfer_block = parent.parentElement;
            const new_block = document.createElement('div');
            advanced_block.appendChild(new_block);

            transfer_block.classList.forEach(className => new_block.classList.add(className));
            new_block.append(parent);
            return;
        }
        parent = parent.parentElement;
    }
}

async function check_data_exists(type) {
    const partKey = wc_user_data?.token ? wc_user_data.token.substring(5, 20) : '';
    const key = type.substring(0, 3).toUpperCase() + partKey;

    const data = get_webcheckups_storage_data(key) ?? await manage_api_data(type, 'get');
    if (['registrations', 'invoices'].includes(type)) {
        if (data?.entryexists) {
            return true;
        } else if (!data?.data || data.data['errors'] || data.data.length < 1 ) {
            return false;
        } else if (data?.data?.attributes || data?.data.length >= 1) {
            set_webcheckups_storage_data(key, {"entryexists": true, "data": [1]});
            return true;
        }
    }
    return false;
}

/* PAGE FUNCTION */
async function check_main_menu() {
    const menu_list = Array.from(document.getElementById('toplevel_page_website_checkups_menu').querySelector('.wp-submenu').querySelectorAll('li'));
    menu_list.shift();

    menu_list.forEach((item, index) => {
        if (!item['id']) {
            item['id'] = 'menu-item-' + (index + 1);
            item.classList.add('d-none');
        }
    });

    if (!wc_user_data['token']) {
        close_loader();
        menu_list[0].classList.remove('d-none');
        menu_list[0].classList.add('d-block');
    } else {
        const registration_exists = await check_data_exists('registrations');
        if (!registration_exists) {
            menu_list[0].classList.remove('d-none');
            menu_list[0].classList.add('d-block');
        } else {
            const notification_data = await manage_api_data('notifications', 'get');
            if (notification_data['data'].length <= 0) {
                menu_list[0].classList.remove('d-none');
                menu_list[1].classList.remove('d-none');
                menu_list[2].classList.add('d-none');
                menu_list[3].classList.add('d-none');
                menu_list[4].classList.add('d-none');
            } else {
                const checkup_data = await manage_api_data('sitecheckups', 'get');
                let active_checkup = false;

                if (checkup_data['data'].length <= 0) {
                    menu_list[0].classList.remove('d-none');
                    menu_list[1].classList.remove('d-none');
                    menu_list[2].classList.remove('d-none');
                    menu_list[3].classList.add('d-none');
                    menu_list[4].classList.add('d-none');

                    menu_list[0].classList.add('d-block');
                    menu_list[1].classList.add('d-block');
                    menu_list[2].classList.add('d-block');
                    menu_list[3].classList.remove('d-block');
                    menu_list[4].classList.remove('d-block');

                } else {
                    for (const single_checkup_data of checkup_data['data']) {
                        if (single_checkup_data['attributes']['active'] === 1) {
                            active_checkup = true;
                            break;
                        }
                    }

                    menu_list[0].classList.remove('d-none');
                    menu_list[1].classList.remove('d-none');
                    menu_list[2].classList.remove('d-none');

                    menu_list[0].classList.add('d-block');
                    menu_list[1].classList.add('d-block');
                    menu_list[2].classList.add('d-block');

                    if (active_checkup) {
                        menu_list[3].classList.remove('d-none');
                        menu_list[4].classList.remove('d-none');

                        menu_list[3].classList.add('d-block');
                        menu_list[4].classList.add('d-block');
                    } else {
                        menu_list[3].classList.add('d-none');
                        menu_list[4].classList.add('d-none');

                        menu_list[3].classList.remove('d-block');
                        menu_list[4].classList.remove('d-block');
                    }
                }
            }
        }
    }
}

function is_contained(value, list) {
    return list.filter(t => t.id==value).length > 0;
}

function getDefaultCountry(allList) {
    let tmpCountry;
    // try country code
    for (let i=0; i<navigator.languages.length; i++) {
        const lang = navigator.languages[i];
        // try country code
        tmpCountry = lang.substring(3).toLowerCase() ?? false;
        if (tmpCountry && is_contained(tmpCountry, allList)) {
            return tmpCountry;
        }

        // try language code
        tmpCountry = lang.substring(0, 2);
        if (tmpCountry && is_contained(tmpCountry, allList)) {
            return tmpCountry;
        }
    }
}

async function registration_page() {
    try {
        const btn_register = document.getElementById('btn_register');
        const dropdown_language_data = {};
        const dropdown_country_data = {};

        const [language_result_popular, language_result_all] = await Promise.all([manage_api_data('languages?filter[popular]=1', 'get'), manage_api_data('languages', 'get')]);
        const [country_result_popular, country_result_all] = await Promise.all([manage_api_data('countries?filter[popular]=1', 'get'), manage_api_data('countries', 'get')]);
        const product_list = await manage_api_data('subproducts', 'get');

        const langDefault = wc_user_data['site_lang'] ?? navigator.language.substring(0, 2);
        const countryDefault = getDefaultCountry(country_result_all.data);

        clear_all_field(wc_form_data['simpleregisters']);

        dropdown_language_data['popular'] = generate_dropdown_list(language_result_popular.data, 'Please choose', 'language', langDefault);
        dropdown_language_data['all'] = generate_dropdown_list(language_result_all.data, 'Please choose', 'language', langDefault);

        dropdown_country_data['popular'] = generate_dropdown_list(country_result_popular.data, 'Please choose', 'name', countryDefault);
        dropdown_country_data['all'] = generate_dropdown_list(country_result_all.data, 'Please choose', 'name', countryDefault);

        let dropdown_language, switch_language, dropdown_country, switch_country, dropdown_subscription,
            dropdown_max, checkbox_auto_renew, checkbox_accept_term;

        wc_form_data['simpleregisters'].forEach((single_form_data, index) => {
            const field_name = single_form_data['field_name'];

            if (field_name === 'companyNameReg') {
                setTimeout(function () {
                    single_form_data['field_item'].focus();
                }, 500);
            }

            if (field_name === 'emailReg') {
                if (wc_user_data['mail'] !== '') {
                    single_form_data['field_item'].value = wc_user_data['mail'];
                }
            }

            if (field_name === 'firstnameReg') {
                if (wc_user_data['first_name'] !== '') {
                    single_form_data['field_item'].value = wc_user_data['first_name'];
                }
            }
            if (field_name === 'lastnameReg') {
                if (wc_user_data['last_name'] !== '') {
                    single_form_data['field_item'].value = wc_user_data['last_name'];
                }
            }
            if (field_name === 'websiteReg') {
                if (wc_user_data['site_address'] !== '') {
                    single_form_data['field_item'].value = wc_user_data['site_address'];
                }
            }

            if (field_name === 'langReg') {
                dropdown_language = single_form_data['field_item'];
            }
            if (field_name === 'switch_langReg') {
                switch_language = single_form_data['field_item'];
            }
            if (field_name === 'countryReg') {
                dropdown_country = single_form_data['field_item'];
            }
            if (field_name === 'switch_countryReg') {
                switch_country = single_form_data['field_item'];
            }
            if (field_name === 'subproductIdReg') {
                dropdown_subscription = single_form_data['field_item'];
            }
            if (field_name === 'maxReg') {
                dropdown_max = single_form_data['field_item'];
            }
            if (field_name === 'autoRenewReg') {
                checkbox_auto_renew = single_form_data['field_item'];
            }
            if (field_name === 'acceptedTermsConditionsReg') {
                checkbox_accept_term = single_form_data['field_item'];

                wc_form_data['simpleregisters'][index]['field_required'] = false;
                wc_form_data['simpleregisters'][index]['detail'] = undefined;
            }
        });

        if (is_contained(langDefault, language_result_popular.data)) {
            dropdown_language.innerHTML = dropdown_language_data['popular'];
            switch_language.checked = false;
        } else {
            dropdown_language.innerHTML = dropdown_language_data['all'];
            switch_language.checked = true;
            switch_language.value = 1;
        }
        toggle_switch(switch_language, dropdown_language, dropdown_language_data['popular'], dropdown_language_data['all']);

        if (is_contained(countryDefault, country_result_popular.data)) {
            dropdown_country.innerHTML = dropdown_country_data['popular'];
            switch_country.checked = false;
        } else {
            dropdown_country.innerHTML = dropdown_country_data['all'];
            switch_country.checked = true;
            switch_country.value = 1;
        }
        toggle_switch(switch_country, dropdown_country, dropdown_country_data['popular'], dropdown_country_data['all']);

        dropdown_subscription.innerHTML = generate_subscription_lst(product_list['data']);
        dropdown_max.innerHTML = generate_subscription_options(product_list['data'], dropdown_subscription.value ?? 1);

        subscription_info(product_list, dropdown_subscription.value);

        dropdown_subscription.addEventListener('change', async function (e) {
            e.preventDefault();
            subscription_info(product_list, this.value);
            dropdown_max.innerHTML = generate_subscription_options(product_list['data'], this.value ?? 1);
        });

        checkbox_auto_renew.value = 1;
        checkbox_accept_term.value = 0;

        checkbox_auto_renew.checked = true;
        checkbox_auto_renew.addEventListener('change', function () {
            this.value = this.checked ? 1 : 0;
        });

        checkbox_accept_term.addEventListener('change', function () {
            this.value = this.checked ? 1 : 0;
        });

        btn_register.addEventListener('click', function (e) {
            e.preventDefault();
            toggle_disable_frm(this);
            remove_all_error_class();

            if (!field_error_check(wc_form_data['simpleregisters'])) {
                const post_data = {};

                if (checkbox_accept_term.value === '1') {
                    wc_form_data['simpleregisters'].forEach(single_form_data => {
                        if (single_form_data['field_type'] !== 'switch') {
                            const field_name = single_form_data['field_name'];

                            if (field_name === 'isAdminPlugin') {
                                post_data[field_name] = '1';
                            } else if (field_name === 'instanceReg') {
                                post_data[field_name] = 'wordpress';
                            } else if (field_name === 'timeZoneReg') {
                                post_data[field_name] = Intl.DateTimeFormat().resolvedOptions().timeZone;
                            } else if (field_name !== undefined) {
                                if (field_name === 'autoRenewReg') {
                                    post_data[field_name] = parseInt(single_form_data['field_item'].value.trim())
                                } else {
                                    post_data[field_name] = single_form_data['field_item'].value.trim();
                                }
                            }
                        }
                    });

                    manage_api_data('simpleregisters', 'post', post_data)
                        .then(result => {
                            if (!result['meta']) {
                                generate_multiple_toast(result['errors']);
                                setTimeout(function () {
                                    toggle_disable_frm(btn_register);
                                    close_loader();
                                }, reactivate_form_time);
                            } else {
                                post_data['token'] = result['data']['attributes']['token'];
                                post_data['notice'] = result['meta']['notice'];
                                post_data['post_type'] = 'user_register';

                                manage_local_data(post_data);
                            }
                        })
                        .catch(error => {
                            console.error('Error:', error);
                        });
                } else {
                    checkbox_accept_term.classList.add('is-invalid');
                    show_toast(false, 'Please Accept Terms and conditions & Privacy Policy');
                    setTimeout(function () {
                        toggle_disable_frm(btn_register);
                    }, reactivate_form_time);
                }

            } else {
                setTimeout(function () {
                    toggle_disable_frm(btn_register);
                }, reactivate_form_time);
            }
        });
    } catch (error) {
        console.error('Error:', error);
    }
}

async function dashboard_page() {
    const dropdown_dashboard_action = document.getElementById('dropdown_dashboard_action');

    dropdown_dashboard_action.addEventListener('change', async function (e) {
        e.preventDefault();
        await load_dashboard_list(this.value.trim(), this.options[this.selectedIndex].text);
    });

    await load_dashboard_list('day', 'last day');
}

async function setting_page() {
    let active_tab = '';

    const setting_tab_list = document.querySelectorAll('button[data-bs-toggle="tab"]')
    const setting_tab_list_arr = Array.from(setting_tab_list);

    for (const setting_tab of setting_tab_list_arr) {
        const index = setting_tab_list_arr.indexOf(setting_tab);
        if (index === 0) {
            active_tab = setting_tab.getAttribute('data-bs-target').split('#')[1];
        }
        setting_tab.addEventListener('shown.bs.tab', async function (event) {
            active_tab = event.target.getAttribute('data-bs-target').split('#')[1];
            await change_tab_setting(active_tab);
        });

        if (wc_user_data['token'].length === 0) {
            if (index !== 0) {
                setting_tab.remove();
            }
        } else {
            const registration_exists = await check_data_exists('registrations');

            if (!registration_exists) {
                setting_tab_list_arr[2].remove();
                setting_tab_list_arr[3].remove();
            } else {
                const invoices_exists = await check_data_exists('invoices');
                if (!invoices_exists) {
                    setting_tab_list_arr[3].remove();
                }
            }
        }
        await change_tab_setting(active_tab);
    }
}

async function notification_page() {

    clear_all_field(wc_form_data['notifications']);

    const btn_add_notifications_modal = document.getElementById('btn_add_notifications_modal');
    btn_popup_manage_notification = document.getElementById('btn_popup_manage_notification');

    const product_list = await manage_api_data('subproducts', 'get');
    const subscription_data = await manage_api_data('subscriptions', 'get');

    let dropdown_notification_type;

    field_error_clear(wc_form_data['notifications']);

    await load_notification_data();

    wc_form_data['notifications'].forEach(single_notification_form_data => {
        const field_name = single_notification_form_data['field_name'];

        if (field_name === 'notificationType') {
            dropdown_notification_type = single_notification_form_data['field_item'];
            dropdown_notification_type.innerHTML = '';

            const tmp_channel_data = product_list['data'].find(item => parseInt(item['id']) === subscription_data['data']['attributes']['subproductId'])['attributes']['channel'];
            generate_filter_dropdown(tmp_channel_data, dropdown_notification_type, single_notification_form_data['values_description']);

            add_tooltip_txt(dropdown_notification_type, dropdown_notification_type.options[dropdown_notification_type.selectedIndex].getAttribute('data-tooltip'));
        }

        single_notification_form_data['field_item'].addEventListener('change', async function (e) {
            e.preventDefault();
            if (single_notification_form_data['field_type'] === 'select' && single_notification_form_data['values_description'] !== undefined) {
                add_tooltip_txt(this, this.options[this.selectedIndex].getAttribute('data-tooltip'));
            }
        });

    });

    btn_add_notifications_modal.addEventListener('click', async function (e) {
        e.preventDefault();
        btn_popup_manage_notification.setAttribute('data-method', 'post');
        btn_popup_manage_notification.removeAttribute('data-id');
        reset_form_field();
        clear_all_form();
        dropdown_notification_type.selectedIndex = 0;
    })

    btn_popup_manage_notification.addEventListener('click', async function (e) {
        e.preventDefault();
        toggle_disable_frm(this);
        remove_all_error_class();

        if (!field_error_check(wc_form_data['notifications'])) {
            const post_data = {};

            wc_form_data['notifications'].forEach(single_notification_form_data => {
                const field_name = single_notification_form_data['field_name'];
                post_data[field_name] = single_notification_form_data['field_item'].value.trim();
            });

            if (this.getAttribute('data-method') === 'patch') {
                post_data['item_id'] = this.getAttribute('data-item_id');
            }

            const notification_post_data = await manage_api_data('notifications', this.getAttribute('data-method'), post_data);

            if (notification_post_data['errors']) {
                generate_multiple_toast(notification_post_data['errors']);
                close_loader();

                toggle_disable_frm(btn_popup_manage_notification);
            } else {
                close_modal('modal_notifications');
                if (!notification_post_data['meta']) {
                    show_toast(true, 'Your data has been processed successfully.', 1);
                } else {
                    show_toast(true, notification_post_data['meta']['notice'], 1);
                    close_loader();
                }

                await load_notification_data().then(() => {
                    reset_form_field();
                    toggle_disable_frm(btn_popup_manage_notification);
                }).catch(error => {
                    console.error('Error:', error);
                })
            }
        } else {
            toggle_disable_frm(btn_popup_manage_notification);
        }
    })
}

async function checkup_page() {
    clear_all_field(wc_form_data['sitecheckups']);
    const btn_add_sitecheckups_modal = document.getElementById('btn_add_sitecheckups_modal');
    btn_popup_manage_checkup = document.getElementById('btn_popup_manage_checkup');

    let field_checkup_name, dropdown_checkup_interval, dropdown_checkup_request_type,
        dropdown_checkup_notification_list,
        field_notify_After, field_wait;

    field_error_clear(wc_form_data['sitecheckups']);

    const product_list = await manage_api_data('subproducts', 'get');
    const subscription_data = await manage_api_data('subscriptions', 'get');
    const notification_data = await manage_api_data('notifications', 'get');

    await load_checkup_data();

    wc_form_data['sitecheckups'].forEach(single_checkup_form_data => {
        const field_name = single_checkup_form_data['field_name'];

        if (field_name === 'name') {
            field_checkup_name = single_checkup_form_data['field_item'];
        }

        if (field_name === 'notificationId') {
            dropdown_checkup_notification_list = single_checkup_form_data['field_item'];
            dropdown_checkup_notification_list.innerHTML = generate_grouped_dropdown_list(notification_data['data'], ['firstnameNotification', 'lastnameNotification', 'notificationValue'], true);
        }

        if (field_name === 'checkInterval') {
            const minInterval = parseInt(subscription_data['data']['attributes']['frequency']);
            const tmp_interval_json_data = product_list['data'].
                find(item => parseInt(item['id']) === subscription_data['data']['attributes']['subproductId'])['attributes']['checkIntervals'].
                filter(item => item.value >= minInterval);

            dropdown_checkup_interval = single_checkup_form_data['field_item'];

            dropdown_checkup_interval.innerHTML = generate_dropdown_list(tmp_interval_json_data, null, 'text', null, false);
        }

        if (field_name === 'requestAs') {
            const request_types = [
                {'id': 'GET', 'name': 'Link availability check (GET)', 'type': 'Common'},
                {'id': 'PING', 'name': 'PING check (ICMP)', 'type': 'Common'},
                {'id': 'TCP', 'name': 'TCP Port connection check', 'type': 'Common'},
                {'id': 'UDP', 'name': 'UDP Port connection check', 'type': 'Common'},
                {'id': 'POST', 'name': 'POST request', 'type': 'Specific'},
                {'id': 'DELETE', 'name': 'DELETE request', 'type': 'Specific'},
                {'id': 'HEAD', 'name': 'HEAD request', 'type': 'Specific'},
                {'id': 'OPTIONS', 'name': 'OPTIONS request', 'type': 'Specific'},
                {'id': 'PATCH', 'name': 'PATCH request', 'type': 'Specific'},
                {'id': 'PUT', 'name': 'PUT request', 'type': 'Specific'},
            ].map(item => ({
                'type': 'request_type',
                'id': item.id,
                'attributes': {
                    'item_name': item.name,
                    'notificationType': item.type + ' request',
                },
            }));

            dropdown_checkup_request_type = single_checkup_form_data['field_item'];
            dropdown_checkup_request_type.innerHTML = generate_grouped_dropdown_list(request_types, ['item_name']);

            dropdown_checkup_request_type.addEventListener('change', function (e) {
                e.preventDefault();
                wc_form_data['sitecheckups'].forEach(inner_single_form_data => {
                    let inner_element = inner_single_form_data['field_item']
                    while (inner_element && inner_element !== document) {
                        if (inner_element.classList.contains('hidden_toggle')) {
                            inner_element.classList[!inner_single_form_data['disable_for'].includes(this.value) ? 'remove' : 'add']('d-none');
                            break;
                        }

                        inner_element = inner_element.parentElement;
                    }
                });
            });
        }

        if (field_name === 'verifySsl' || field_name === 'active') {
            single_checkup_form_data['field_item'].value = false;
            single_checkup_form_data['field_item'].checked = false;
            single_checkup_form_data['field_item'].addEventListener('change', function () {
                this.value = this.checked ? 1 : 0;
            });
        }

        if (field_name === 'notifyAfter') {
            field_notify_After = single_checkup_form_data['field_item'];
        }

        if (field_name === 'wait') {
            field_wait = single_checkup_form_data['field_item'];
        }

        if (single_checkup_form_data['field_advanced'] === 1) {
            generate_advance_block('sitecheckups_block_advanced', 'sitecheckups_block_advanced_btn', btn_add_sitecheckups_modal.dataset.bsTarget, single_checkup_form_data['field_item']);
        }
    });

    btn_add_sitecheckups_modal.addEventListener('click', async function (e) {
        e.preventDefault();
        btn_popup_manage_checkup.setAttribute('data-method', 'post');
        btn_popup_manage_checkup.removeAttribute('data-id');
        reset_form_field();
        clear_all_form();
        dropdown_checkup_interval.selectedIndex = 0;
        dropdown_checkup_request_type.selectedIndex = 0;
        dropdown_checkup_notification_list.value = '';
        field_notify_After.value = 2;
        field_wait.value = 3;
        setTimeout(function () {
            field_checkup_name.focus();
            if (jQuery('#noChecksFrom, #noChecksUntil')) {
                jQuery('#noChecksFrom, #noChecksUntil').mask('00:00');
            }
        }, 500);
        document.getElementById('sitecheckups_block_advanced').classList.add('d-none');

        wc_form_data['sitecheckups'].forEach(single_checkup_form_data => {
            let element = single_checkup_form_data['field_item'];
            if (single_checkup_form_data['field_name'] === 'verifySsl' || single_checkup_form_data['field_name'] === 'active') {
                single_checkup_form_data['field_item'].value = 0;
                single_checkup_form_data['field_item'].checked = false;
            }

            while (element && element !== document) {
                if (element.classList.contains('hidden_toggle')) {
                    element.classList[!single_checkup_form_data['disable_for'].includes(dropdown_checkup_request_type.value) ? 'remove' : 'add']('d-none');
                    break;
                }
                element = element.parentElement;
            }
        });
    })

    btn_popup_manage_checkup.addEventListener('click', async function (e) {
            e.preventDefault();
            toggle_disable_frm(this);
            remove_all_error_class();
            let tmp_checkup_from_data = [];

            wc_form_data['sitecheckups'].forEach(single_checkup_form_data => {
                let element = single_checkup_form_data['field_item'];
                let isHidden = false;

                while (element && element !== document) {
                    if (element.classList.contains('d-none')) {
                        isHidden = true;
                        break;
                    }
                    element = element.parentElement;
                }

                if (!isHidden) {
                    tmp_checkup_from_data.push(single_checkup_form_data);
                    setTimeout(function () {
                        if (jQuery('#noChecksFrom, #noChecksUntil')) {
                            jQuery('#noChecksFrom, #noChecksUntil').mask('00:00');
                        }
                    }, 500);
                }
            });

            if (!field_error_check(tmp_checkup_from_data)) {
                const post_data = {};

                wc_form_data['sitecheckups'].forEach(single_checkups_form_data => {
                    const field_name = single_checkups_form_data['field_name'];
                    post_data[field_name] = single_checkups_form_data['field_item'].value.trim();

                    if (field_name === 'notificationId') {
                        post_data[field_name] = Array.from(single_checkups_form_data['field_item'].options).filter(option => option.selected).map(option => option.value);
                    }
                    if (field_name === 'checkInterval' || field_name === 'wait' || field_name === 'responseCode' || field_name === 'verifySsl' || field_name === 'notifyAfter' || field_name === 'active') {
                        post_data[field_name] = parseInt(single_checkups_form_data['field_item'].value.trim())
                    }

                    if (field_name === 'verifySsl' || field_name === 'active') {
                        if (single_checkups_form_data['field_item'].value.trim() === '') {
                            single_checkups_form_data['field_item'].value = 0;
                            post_data[field_name] = single_checkups_form_data['field_item'].value.trim();
                        }
                    }

                    if (field_name === 'responseText' || field_name === 'postData' || field_name === 'headers') {
                        post_data[field_name] = single_checkups_form_data['field_item'].value.trim() === '' ? null : single_checkups_form_data['field_item'].value.trim();
                    }
                });

                if (this.getAttribute('data-method') === 'patch') {
                    post_data['item_id'] = this.getAttribute('data-item_id');
                }

                toggle_disable_frm(btn_popup_manage_checkup);
                const site_checkups_post_data = await manage_api_data('sitecheckups', this.getAttribute('data-method'), post_data);

                if (site_checkups_post_data['errors']) {
                    generate_multiple_toast(site_checkups_post_data['errors']);
                    close_loader();
                } else {
                    close_modal('modal_sitecheckups');
                    if (!site_checkups_post_data['meta']) {
                        show_toast(true, 'Your data has been processed successfully.', 1);
                    } else {
                        show_toast(true, site_checkups_post_data['meta']['notice'], 1);
                        close_loader();
                    }


                    await load_checkup_data().then(() => {
                        reset_form_field();
                        toggle_disable_frm(btn_popup_manage_checkup);
                    }).catch(error => {
                        console.error('Error:', error);
                    })
                }
            } else {
                toggle_disable_frm(btn_popup_manage_checkup);
            }
        }
    )
}

async function checkup_details_page() {
    const dropdown_checkup_result_period = document.getElementById('dropdown_checkup_result_period');
    const dropdown_checkup_list = document.getElementById('dropdown_checkup_list');

    const checkup_data = await manage_api_data('sitecheckups', 'get');

    if (checkup_data['data'].length > 0) {
        const active_checkups = checkup_data['data'].filter(item => item['attributes']['active'] === 1);

        dropdown_checkup_list.innerHTML = generate_dropdown_list(active_checkups, null, 'name');
        dropdown_checkup_list.selectedIndex = 0;
        if (checkup_details_id !== 0) {
            dropdown_checkup_list.value = checkup_details_id;
            dropdown_checkup_result_period.value = checkup_time_id;
        } else {
            dropdown_checkup_list.selectedIndex = 0;
            dropdown_checkup_result_period.selectedIndex = 0;
        }

        dropdown_checkup_result_period.addEventListener('change', async function (e) {
            e.preventDefault();
            showGraphic(parseInt(dropdown_checkup_list.value.trim()), this.value.trim(), 'details_chart');
            await load_checkup_details(dropdown_checkup_list.value.trim(), this.value.trim());
        });

        dropdown_checkup_list.addEventListener('change', async function (e) {
            e.preventDefault();
            showGraphic(parseInt(this.value.trim()), dropdown_checkup_result_period.value.trim(), 'details_chart');
            await load_checkup_details(parseInt(this.value.trim()), dropdown_checkup_result_period.value.trim());
        });

        showGraphic(parseInt(dropdown_checkup_list.value.trim()), dropdown_checkup_result_period.value.trim(), 'details_chart');

        await load_checkup_details(parseInt(dropdown_checkup_list.value.trim()), dropdown_checkup_result_period.value.trim());

    }
}

async function dashboard_page() {
    const dropdown_dashboard_action = document.getElementById('dropdown_dashboard_action');

    dropdown_dashboard_action.addEventListener('change', async function (e) {
        e.preventDefault();
        await load_dashboard_list(this.value.trim(), this.options[this.selectedIndex].text);
    });

    await load_dashboard_list('day', 'last day');
}

async function change_tab_setting(active_tab) {
    clear_all_toast();
    clear_all_form();
    if (active_tab === 'tab_user_details') {
        const language_data = await manage_api_data('languages', 'get');
        const users_data = await manage_api_data('users', 'get');
        const users_fields = await prepare_form_fields('users') ?? wc_form_data['users'];

        users_fields.forEach(single_user_form_data => {
            const field_name = single_user_form_data['field_name'];

            if (field_name === 'lang') {
                single_user_form_data['field_item'].innerHTML = generate_dropdown_list(language_data['data'], null, 'language');
            }

            single_user_form_data['field_item'].value = users_data['data']['attributes'][field_name];

        });

        document.getElementById('tab_user_details').querySelector('fieldset').disabled = true;
    }

    if (active_tab === 'tab_registration') {
        let dropdown_country, switch_country, checkbox_accept_term;
        const btn_update_registration = document.getElementById('btn_update_registration');

        const dropdown_country_data = {};
        const [country_result_popular, country_result_all] = await Promise.all([manage_api_data('countries?filter[popular]=1', 'get'), manage_api_data('countries', 'get')]);

        dropdown_country_data['popular'] = generate_dropdown_list(country_result_popular.data, 'Please choose', 'name');
        dropdown_country_data['all'] = generate_dropdown_list(country_result_all.data, 'Please choose', 'name');

        const registration_data = await manage_api_data('registrations', 'get');

        wc_form_data['registrations'].forEach((single_registration_form_data, index) => {
            const field_name = single_registration_form_data['field_name'];

            if (field_name === 'country') {
                dropdown_country = single_registration_form_data['field_item'];
                dropdown_country.innerHTML = dropdown_country_data['all'];
            }

            if (field_name === 'switch_country') {
                switch_country = single_registration_form_data['field_item'];
            }

            if (field_name === 'acceptedTermsConditions') {
                checkbox_accept_term = single_registration_form_data['field_item'];

                wc_form_data['registrations'][index]['field_required'] = false;
                wc_form_data['registrations'][index]['detail'] = undefined;
            }

            single_registration_form_data['field_item'].value = registration_data['data']['attributes'][field_name];
        });

        switch_country.checked = true;

        toggle_switch(switch_country, dropdown_country, dropdown_country_data['popular'], dropdown_country_data['all']);

        checkbox_accept_term.addEventListener('change', function () {
            this.value = this.checked ? 1 : 0;
        });

        btn_update_registration.addEventListener('click', async function (e) {
            e.preventDefault();
            toggle_disable_frm(this);
            remove_all_error_class();

            if (!field_error_check(wc_form_data['registrations'])) {
                const post_data = {
                    "item_id": registration_data['data']['id']
                };
                if (checkbox_accept_term.value === '1') {
                    wc_form_data['registrations'].forEach(single_registration_form_data => {
                        if (single_registration_form_data['field_type'] !== 'switch') {
                            const field_name = single_registration_form_data['field_name'];
                            post_data[field_name] = single_registration_form_data['field_item'].value.trim();
                        }
                    });
                    const registration_result = await manage_api_data('registrations', 'patch', post_data);

                    if (registration_result['errors']) {
                        generate_multiple_toast(registration_result['errors']);
                        close_loader();
                    } else {
                        if (!registration_result['meta']) {
                            show_toast(true, 'User Details Updated Successfully.', 1);
                            checkbox_accept_term.checked = false;
                        } else {
                            show_toast(true, registration_result['meta']['notice'], 1);
                            setTimeout(function () {
                                location.reload();
                            }, 2000);
                        }
                        close_loader();
                    }

                    toggle_disable_frm(btn_update_registration);

                } else {
                    checkbox_accept_term.classList.add('is-invalid');
                    show_toast(false, 'Please Accept Terms and conditions & Privacy Policy');
                    setTimeout(function () {
                        toggle_disable_frm(btn_update_registration);
                    }, reactivate_form_time);
                }
            } else {
                toggle_disable_frm(btn_update_registration);
            }
        });
    }

    if (active_tab === 'tab_subscription') {
        const btn_update_subscription = document.getElementById('btn_update_subscription');
        let dropdown_subscription, checkbox_auto_renew, dropdown_max;
        const product_list = await manage_api_data('subproducts', 'get');
        const subscription_data = await manage_api_data('subscriptions', 'get');

        wc_form_data['subscriptions'].forEach(single_subscription_form_data => {
            const field_name = single_subscription_form_data['field_name'];

            if (field_name === 'subproductId') {
                dropdown_subscription = single_subscription_form_data['field_item'];
                dropdown_subscription.innerHTML = generate_subscription_lst(product_list['data']);
                subscription_info(product_list, subscription_data['data']['attributes'][field_name] ?? 1);
            }

            if (field_name === 'max') {
                dropdown_max = single_subscription_form_data['field_item'];
                dropdown_max.innerHTML = generate_subscription_options(
                    product_list['data'],
                    subscription_data?.data?.attributes?.subproductId ?? 1,
                    subscription_data?.data?.attributes?.max ?? 1
                );
            }

            if (field_name === 'autoRenew') {
                checkbox_auto_renew = single_subscription_form_data['field_item'];
            }

            single_subscription_form_data['field_item'].value = subscription_data['data']['attributes'][field_name];
            single_subscription_form_data['field_item'].checked = subscription_data['data']['attributes'][field_name];
        });

        dropdown_subscription.addEventListener('change', async function (e) {
            e.preventDefault();
            subscription_info(product_list, this.value ?? 1);
            dropdown_max.innerHTML = generate_subscription_options(
                product_list['data'],
                this.value ?? 1,
                subscription_data?.data?.attributes?.max);
        });

        btn_update_subscription.addEventListener('click', async function (e) {
            e.preventDefault();
            toggle_disable_frm(this);
            remove_all_error_class();

            const post_data = {
                "item_id": subscription_data['data']['id'],
            };

            wc_form_data['subscriptions'].forEach(single_registration_form_data => {
                if (single_registration_form_data['field_type'] !== 'switch') {
                    const field_name = single_registration_form_data['field_name'];
                    post_data[field_name] = single_registration_form_data['field_item'].value.trim();
                }
            });

            const subscription_result = await manage_api_data('subscriptions', 'patch', post_data);

            if (!subscription_result['meta']) {
                generate_multiple_toast(subscription_result['errors']);
                show_toast(true, 'Your data has been processed successfully.', 1);
                close_loader();
            } else {
                show_toast(true, 'Your data has been processed successfully.', 1);
            }

            close_loader();
            toggle_disable_frm(this);
        });
    }

    if (active_tab === 'tab_invoice') {
        const element_setting_plan_name = document.getElementById('element_setting_plan_name');
        const element_setting_auto_renew = document.getElementById('element_setting_auto_renew');
        const element_setting_invoice_date = document.getElementById('element_setting_invoice_date');
        const tbl_setting_invoice_list = document.getElementById('tbl_setting_invoice_list');
        const pay_invoice_text = document.getElementById('block_settings_info_text');
        const invoice_data = await manage_api_data('invoices', 'get');

        if (invoice_data['data'].length > 0) {
            const product_list = await manage_api_data('subproducts', 'get');

            const latest_invoice_data = invoice_data['data'][Object.keys(invoice_data['data']).length - 1];
            const subscription_data = await manage_api_data('subscriptions', 'get');

            element_setting_plan_name.innerText = get_subscription_name(product_list, latest_invoice_data['attributes']['subproductId']);
            element_setting_invoice_date.innerHTML = format_date(latest_invoice_data['attributes']['end']);

            element_setting_auto_renew.innerText = subscription_data['data']['attributes']['autoRenew'] ? 'with' : 'without';

            const invoice_column_data = await fetch_template('tbl_column/template_tbl_column_settings_invoice.html');
            tbl_setting_invoice_list.querySelector('tbody').innerHTML = '';
            for (const invoice_tbl_data of invoice_data['data']) {
                tbl_setting_invoice_list.querySelector('tbody').innerHTML += invoice_column_data.replace('{{bill_number}}', 'W' + invoice_tbl_data['id'])
                    .replace('{{invoice_url}}', wc_site_url + invoice_tbl_data['attributes']['link'])
                    .replace('{{invoice_date}}', format_date(invoice_tbl_data['attributes']['invoiceDate']))
                    .replace('{{invoice_total}}', format_num(invoice_tbl_data['attributes']['total'], 2))
                    .replace('{{subscription_plan}}', get_subscription_name(product_list, invoice_tbl_data['attributes']['subproductId']))
                    .replace('{{status}}', invoice_tbl_data['attributes']['status'])
                    .replace('{{service_period}}', format_date(invoice_tbl_data['attributes']['start']) + ' - ' + format_date(invoice_tbl_data['attributes']['end']))

                if (invoice_tbl_data['attributes']['status'] == 'open' && invoice_tbl_data['attributes']['paymentLink']) {
                    pay_invoice_text.innerHTML = '<br />You can pay your open invoice #W' + invoice_tbl_data['id'] +
                        ' <a href="' + invoice_tbl_data['attributes']['paymentLink'] + '" target="_blank">here</a>.'
                }
            }
        }
    }

    close_loader();
}

async function load_notification_data() {
    const dropdown_notification_action = document.getElementById('dropdown_notification_action');
    const btn_notification_action = document.getElementById('btn_notification_action');
    const dropdown_notification_filter = document.getElementById('dropdown_notification_filter');
    const btn_notification_filter = document.getElementById('btn_notification_filter');
    const field_notification_search = document.getElementById('field_notification_search');
    const btn_notification_search = document.getElementById('btn_notification_search');
    const tbl_notification = document.getElementById('tbl_notification');
    const notification_checkbox_select_all = document.getElementById('notification_checkbox_select_all');

    notification_checkbox_select_all.checked = false;

    setTimeout(async function () {
        tbl_notification.querySelector('tbody').innerHTML = '';
        const notification_data = await manage_api_data('notifications', 'get');

        if (notification_data['data'].length > 0) {
            filter_dropdown_items(notification_data['data'], dropdown_notification_filter, 'notificationType');

            const notification_tbl_column = await fetch_template('tbl_column/template_tbl_column_notifications.html');
            tbl_notification.querySelector('tbody').innerHTML = '';
            for (const notification_tbl_data of notification_data['data']) {
                tbl_notification.querySelector('tbody').innerHTML += notification_tbl_column.replace('{{item_id}}', notification_tbl_data['id'])
                    .replace('{{user_name}}', (notification_tbl_data['attributes']['firstnameNotification'] || '') + ' ' + (notification_tbl_data['attributes']['lastnameNotification'] || ''))
                    .replace('{{notification_type}}', notification_tbl_data['attributes']['notificationType'])
                    .replace('{{notification_value}}', notification_tbl_data['attributes']['notificationValue'])
                    .replace('{{verify_mark}}', notification_tbl_data['attributes']['verifiedAt'] !== null ? 'tick_mark' : 'close_mark')
            }

            const btn_notification_edit = document.querySelectorAll('.btn_notification_edit');
            const btn_notification_delete = document.querySelectorAll('.btn_notification_delete');
            const notification_check_box_single = document.querySelectorAll('.notification_check_box_single');

            toggle_all_check_box(notification_checkbox_select_all, notification_check_box_single);

            btn_notification_action.addEventListener('click', function (e) {
                e.preventDefault();
                let tmp_data = [];
                if (dropdown_notification_action.value.trim() === 'remove') {
                    notification_check_box_single.forEach(notification_single_checkbox => {
                        if (notification_single_checkbox.checked) {
                            tmp_data.push(parseInt(notification_single_checkbox.closest('tr').getAttribute('data-item_id')));
                        }
                    });
                }

                if (tmp_data.length < 1) {
                    show_toast(false, 'No notifications selected', 1);
                    return;
                }

                const post_data = {
                    bulk_select: true,
                    bulk_type: dropdown_notification_action.value.trim(),
                    notificationId: tmp_data
                };

                manage_api_data('notifications', 'post', post_data)
                    .then(result => {
                        if (!result) {
                            show_toast(true, 'All Notifications Deleted Successfully.', 1);
                            load_notification_data();
                        }
                    })
                    .catch(error => {
                        console.error('Error:', error);
                    });
            });

            btn_notification_filter.addEventListener('click', function () {
                search_tbl(tbl_notification, dropdown_notification_filter.value.trim().toLowerCase(), [2]);
            });

            field_notification_search.addEventListener('input', function () {
                if (this.value === '') {
                    search_tbl(tbl_notification, '', [1, 3]);
                }
            });

            btn_notification_search.addEventListener('click', function () {
                search_tbl(tbl_notification, field_notification_search.value.trim().toLowerCase(), [1, 3]);
            });

            btn_notification_edit.forEach(notification_edit_btn => {
                notification_edit_btn.addEventListener('click', async function (e) {
                    e.preventDefault();
                    clear_all_field(wc_form_data['notifications']);
                    clear_all_form();
                    const tmp_item_id = this.closest('tr').getAttribute('data-item_id');
                    const single_notification_details = await manage_api_data('notifications/' + tmp_item_id, 'get');

                    if (single_notification_details['data'] !== null) {
                        reset_form_field();
                        btn_popup_manage_notification.setAttribute('data-method', 'patch');
                        btn_popup_manage_notification.setAttribute('data-item_id', tmp_item_id);

                        wc_form_data['notifications'].forEach(single_notification_form_data => {
                            const field_name = single_notification_form_data['field_name'];
                            single_notification_form_data['field_item'].value = single_notification_details['data']['attributes'][field_name];
                        });
                    }
                    close_loader();
                });
            });

            btn_notification_delete.forEach(btn_notification_delete_btn => {
                btn_notification_delete_btn.addEventListener('click', async function (e) {
                    e.preventDefault();
                    if (confirm_window('Are you Sure Delete This Notification?')) {
                        const post_data = {
                            item_id: this.closest('tr').getAttribute('data-item_id')
                        };

                        const notification_delete_data = await manage_api_data('notifications', 'delete', post_data);

                        if (notification_delete_data) {
                            show_toast(true, 'Notification Deleted Successfully.', 1);
                        } else {
                            show_toast(true, 'Notification Deleted Successfully.', 1);
                        }

                        await load_notification_data();
                    }
                });
            });
        }
        notification_checkbox_select_all.checked = false;
        dropdown_notification_action.value = '';
        await check_main_menu();
        close_loader();
    }, 500);

}

async function load_checkup_data() {
    const dropdown_checkup_action = document.getElementById('dropdown_checkup_action');
    const btn_checkup_action = document.getElementById('btn_checkup_action');
    const dropdown_checkup_filter = document.getElementById('dropdown_checkup_filter');
    const btn_checkup_filter = document.getElementById('btn_checkup_filter');
    const field_checkup_search = document.getElementById('field_checkup_search');
    const btn_checkup_search = document.getElementById('btn_checkup_search');
    const tbl_checkup = document.getElementById('tbl_checkup');
    const checkup_checkbox_select_all = document.getElementById('checkup_checkbox_select_all');

    checkup_checkbox_select_all.checked = false;

    setTimeout(async function () {
        tbl_checkup.querySelector('tbody').innerHTML = '';
        const checkup_data = await manage_api_data('sitecheckups', 'get');

        if (checkup_data['data'].length > 0) {
            filter_dropdown_items(checkup_data['data'], dropdown_checkup_filter, 'requestAs');

            const checkup_tbl_column = await fetch_template('tbl_column/template_tbl_column_checkups.html');
            tbl_checkup.querySelector('tbody').innerHTML = '';
            for (const checkup_tbl_data of checkup_data['data']) {
                tbl_checkup.querySelector('tbody').innerHTML += checkup_tbl_column.replace('{{item_id}}', checkup_tbl_data['id'])
                    .replace('{{checkup_type}}', checkup_tbl_data['attributes']['requestAs'])
                    .replace('{{checkup_name}}', checkup_tbl_data['attributes']['name'])
                    .replace('{{checkup_url}}', checkup_tbl_data['attributes']['link'])
                    .replace('{{checkup_interval}}', format_time(checkup_tbl_data['attributes']['checkInterval']))
                    .replace('{{active_mark}}', checkup_tbl_data['attributes']['active'] !== 0 ? 'tick_mark' : 'close_mark')
            }

            const btn_checkup_edit = document.querySelectorAll('.btn_checkup_edit');
            const btn_checkup_delete = document.querySelectorAll('.btn_checkup_delete');
            const checkup_check_box_single = document.querySelectorAll('.checkup_check_box_single');

            toggle_all_check_box(checkup_checkbox_select_all, checkup_check_box_single);

            btn_checkup_action.addEventListener('click', function (e) {
                e.preventDefault();
                let tmp_data = [];
                if (dropdown_checkup_action.value.trim() === 'remove') {
                    tmp_data = [];
                    checkup_check_box_single.forEach(checkup_single_checkbox => {
                        if (checkup_single_checkbox.checked) {
                            tmp_data.push(parseInt(checkup_single_checkbox.closest('tr').getAttribute('data-item_id')));
                        }
                    });
                }

                if (dropdown_checkup_action.value.trim() === 'deactivate') {
                    tmp_data = [];
                    checkup_check_box_single.forEach(checkup_single_checkbox => {
                        if (checkup_single_checkbox.checked) {
                            tmp_data.push(checkup_single_checkbox.closest('tr').getAttribute('data-item_id'));
                        }
                    });
                }

                if (tmp_data.length < 1) {
                    show_toast(false, 'No checkups selected', 1);
                    return;
                }

                const post_data = {
                    bulk_select: true,
                    bulk_type: dropdown_checkup_action.value.trim(),
                    checkupId: tmp_data
                };

                manage_api_data('sitecheckups', 'post', post_data)
                    .then(result => {
                        if (!result) {
                            show_toast(true, 'All checkups ' + dropdown_checkup_action.value.trim() + ' successfully.', 1);
                            load_checkup_data();
                        }
                    })
                    .catch(error => {
                        console.error('Error:', error);
                    });
            });

            btn_checkup_filter.addEventListener('click', function () {
                search_tbl(tbl_checkup, dropdown_checkup_filter.value.trim().toLowerCase(), [1]);
            });

            field_checkup_search.addEventListener('input', function () {
                if (this.value === '') {
                    search_tbl(tbl_checkup, '', [2, 3]);
                }
            });

            btn_checkup_search.addEventListener('click', function () {
                search_tbl(tbl_checkup, field_checkup_search.value.trim().toLowerCase(), [2, 3]);
            });

            btn_checkup_edit.forEach(checkup_edit_btn => {
                checkup_edit_btn.addEventListener('click', async function (e) {
                    e.preventDefault();

                    const tmp_item_id = this.closest('tr').getAttribute('data-item_id');
                    const single_checkup_details = await manage_api_data('sitecheckups/' + tmp_item_id, 'get');

                    if (single_checkup_details['data'] !== null) {
                        reset_form_field();
                        clear_all_form();
                        btn_popup_manage_checkup.setAttribute('data-method', 'patch');
                        btn_popup_manage_checkup.setAttribute('data-item_id', tmp_item_id);

                        const single_checkup_details_keys = Object.keys(single_checkup_details['data']['attributes']);

                        wc_form_data['sitecheckups'].forEach(single_site_checkups_form_data => {
                            const field_name = single_site_checkups_form_data['field_name'];
                            single_site_checkups_form_data['field_item'].value = single_checkup_details['data']['attributes'][field_name];

                            if (field_name === 'notificationId') {
                                single_checkup_details['data']['attributes'][field_name].forEach(value => single_site_checkups_form_data['field_item'].querySelector(`option[value='${value}']`).selected = true);
                            }

                            if (field_name === 'verifySsl' || field_name === 'active') {
                                single_site_checkups_form_data['field_item'].checked = parseInt(single_checkup_details['data']['attributes'][field_name]) === 1;
                            }

                            if (single_checkup_details_keys.includes(single_site_checkups_form_data['field_name'])) {
                                let element = single_site_checkups_form_data['field_item'];
                                while (element && element !== document) {
                                    if (element.classList.contains('hidden_toggle')) {
                                        element.classList[!single_site_checkups_form_data['disable_for'].includes(single_checkup_details['data']['attributes']['requestAs']) ? 'remove' : 'add']('d-none');
                                        break;
                                    }
                                    element = element.parentElement;
                                }
                            }
                        });
                    }
                    close_loader();
                });
            });

            btn_checkup_delete.forEach(btn_checkup_delete_btn => {
                btn_checkup_delete_btn.addEventListener('click', async function (e) {
                    e.preventDefault();
                    if (confirm_window('Are you Sure Delete This checkup?')) {
                        const post_data = {
                            item_id: this.closest('tr').getAttribute('data-item_id')
                        };

                        const checkup_delete_data = await manage_api_data('sitecheckups', 'delete', post_data);

                        if (checkup_delete_data) {
                            show_toast(true, 'Checkup Deleted Successfully.', 1);
                        } else {
                            show_toast(true, 'Checkup Deleted Successfully.', 1);
                        }
                        await load_checkup_data();
                    }
                });
            });
        }
        checkup_checkbox_select_all.checkbox = false;
        dropdown_checkup_action.value = '';
        await check_main_menu();
        close_loader();
    }, 500);
}

async function load_checkup_details(checkup_id, checkup_period) {
    const block_no_item = document.querySelector('.block_no_item');
    const block_checkup_details = document.getElementById('block_checkup_details');
    const tbl_checkup_result_slowest_response = document.getElementById('tbl_checkup_result_slowest_response');
    const tbl_checkup_result_last_failure = document.getElementById('tbl_checkup_result_last_failure');
    const tbl_checkup_result_fastest_response = document.getElementById('tbl_checkup_result_fastest_response');

    const block_slowest_responses = document.getElementById('block_slowest_responses');
    const block_fastest = document.getElementById('block_fastest');
    const block_last_failures = document.getElementById('block_last_failures');

    const checkup_details = await manage_api_data('checkupdetails/' + checkup_id + '/' + checkup_period, 'get');

    const block_checkup_result_url = document.getElementById('block_checkup_result_url');
    const block_checkup_result_interval = document.getElementById('block_checkup_result_interval');
    const block_checkup_result_last_check_checkup_time = document.getElementById('block_checkup_result_last_check_checkup_time');
    const block_checkup_result_online_since = document.getElementById('block_checkup_result_online_since');

    const block_checkup_result_ssl_ssl_issuer = document.getElementById('block_checkup_result_ssl_ssl_issuer');
    const block_checkup_result_ssl_cipher_protocol = document.getElementById('block_checkup_result_ssl_cipher_protocol');
    const block_checkup_result_ssl_valid_from_to = document.getElementById('block_checkup_result_ssl_valid_from_to');

    const block_checkup_result_dns_error = document.getElementById('block_checkup_result_dns_error');
    const block_checkup_result_ip_address = document.getElementById('block_checkup_result_ip_address');
    const block_checkup_result_minmax_time = document.getElementById('block_checkup_result_minmax_time');
    const block_checkup_result_domain_expires = document.getElementById('block_checkup_result_domain_expires');
    const block_checkup_successful_failed_checkup = document.getElementById('block_checkup_successful_failed_checkup');
    const details_attr = checkup_details['data']['attributes'];

    if (checkup_details['data'] !== undefined) {
        block_checkup_details.classList.remove('d-none');
        block_no_item.classList.add('d-none');

        const details_link = details_attr['link'];
        const details_check_interval = details_attr['checkInterval'];
        const details_last_check_performed = details_attr['lastCheckupPerformed'];
        const lastCheckupDescription = details_attr['lastCheckupDescription'];
        const details_online_since = details_attr['onlineSince'];

        const details_ssl_status = details_attr['sslStatus'];
        const details_ssl_cipher = details_attr['sslCipher'];
        const details_ssl_valid_from = details_attr['sslValidFrom'];
        const details_ssl_valid_till = details_attr['sslValidTill'];
        const details_dns_error = details_attr['DNSError'];

        const details_ip = details_attr['ip'];
        const details_fastest_checkup = details_attr['fastestCheckups'];
        const details_slowest_checkup = details_attr['slowestCheckups'];
        const details_last_errors = details_attr['failedList'];

        const details_domain_expire = details_attr['domainExpires'];
        const details_successful_checkup = details_attr['checkupsOk'];
        const details_failed_checkup = details_attr['checkupsFailed'];
        document.getElementById('lastCheckupStatus').className = details_attr['lastCheckupStatus'] ?
            'status_ok' : 'status_err';

        block_checkup_result_url.innerText = details_link;
        block_checkup_result_interval.innerText = format_time(details_check_interval);
        block_checkup_result_last_check_checkup_time.innerText = (details_last_check_performed ? format_date(details_last_check_performed) + ', ' : '') + lastCheckupDescription;
        block_checkup_result_online_since.innerText = details_online_since ? format_date(details_online_since) : '-';
        block_checkup_result_ssl_ssl_issuer.innerText = details_ssl_status;

        block_checkup_result_ssl_cipher_protocol.innerText = details_ssl_cipher;

        block_checkup_result_ssl_valid_from_to.innerText = format_date(details_ssl_valid_from) + ' - ' + format_date(details_ssl_valid_till);
        block_checkup_result_dns_error.innerText = details_dns_error;

        block_checkup_result_ip_address.innerText = details_ip;
        block_checkup_result_domain_expires.innerText = format_date(details_domain_expire);
        block_checkup_successful_failed_checkup.innerText = details_successful_checkup + ' successful, ' + details_failed_checkup + ' failed';

        let details_minmax_checkup = false;
        if (details_fastest_checkup !== undefined && details_slowest_checkup !== undefined && details_fastest_checkup != details_slowest_checkup) {
            block_checkup_result_minmax_time.innerText = 'Slowest: ' + get_most_recent_time(details_slowest_checkup, 'slow')['time'] + ' sec.' +
                ', fastest: ' + get_most_recent_time(details_fastest_checkup, 'fast')['time'] + ' sec.';
            details_minmax_checkup = true;
        }

        const shouldShowSSLValid = details_ssl_valid_from && details_ssl_valid_till;

        block_checkup_result_url.closest('li').classList.toggle('d-none', details_link === '');
        block_checkup_result_interval.closest('li').classList.toggle('d-none', details_check_interval === '');
        block_checkup_result_online_since.closest('li').classList.toggle('d-none', details_online_since === '-');
        block_checkup_result_ssl_ssl_issuer.closest('li').classList.toggle('d-none', !details_ssl_status);

        block_checkup_result_ssl_cipher_protocol.closest('li').classList.toggle('d-none', !details_ssl_cipher);

        block_checkup_result_ssl_valid_from_to.closest('li').classList.toggle('d-none', !shouldShowSSLValid);
        block_checkup_result_dns_error.closest('li').classList.toggle('d-none', details_dns_error === '' || details_dns_error === undefined || details_dns_error === false);

        block_checkup_result_ip_address.closest('li').classList.toggle('d-none', details_ip === '' || details_ip === undefined);
        block_checkup_result_domain_expires.closest('li').classList.toggle('d-none', details_domain_expire === '' || details_domain_expire === undefined);
        block_checkup_result_minmax_time.closest('li').classList.toggle('d-none', !details_minmax_checkup);

        if (details_slowest_checkup !== undefined && details_last_check_performed !== undefined) {
            block_slowest_responses.classList.remove('d-none');
            const checkup_slowest_checkup_tbl_column = await fetch_template('tbl_column/template_tbl_column_checkup_results.html');
            tbl_checkup_result_slowest_response.querySelector('tbody').innerHTML = '';

            details_slowest_checkup.forEach((checkup_details_slowest_checkup_data, index) => {
                tbl_checkup_result_slowest_response.querySelector('tbody').innerHTML += checkup_slowest_checkup_tbl_column
                    .replace('{{index_item}}', (index + 1))
                    .replace('{{checkup_results_date}}', format_date(checkup_details_slowest_checkup_data['performed']))
                    .replace('{{checkup_results_result}}', checkup_details_slowest_checkup_data['time'] + ' sec');
            });
        } else {
            tbl_checkup_result_slowest_response.querySelector('tbody').innerHTML = '';
        }

        if (details_last_errors !== undefined) {
            block_last_failures.classList.remove('d-none');
            const checkup_last_error_tbl_column = await fetch_template('tbl_column/template_tbl_column_checkup_results.html');
            tbl_checkup_result_last_failure.querySelector('tbody').innerHTML = '';

            details_last_errors.forEach((checkup_details_last_error_data, index) => {
                const dateRow = getTimespan(checkup_details_last_error_data['from'], checkup_details_last_error_data['until']);
                tbl_checkup_result_last_failure.querySelector('tbody').innerHTML += checkup_last_error_tbl_column
                    .replace('{{index_item}}', (index + 1))
                    .replace('{{checkup_results_date}}', dateRow)
                    .replace('{{checkup_results_result}}', checkup_details_last_error_data['description']);
            });
        } else {
            tbl_checkup_result_last_failure.querySelector('tbody').innerHTML = '';
        }

        if (details_fastest_checkup !== undefined && details_last_check_performed !== undefined) {
            block_fastest.classList.remove('d-none');
            const checkup_fastest_checkup_tbl_column = await fetch_template('tbl_column/template_tbl_column_checkup_results.html');
            tbl_checkup_result_fastest_response.querySelector('tbody').innerHTML = '';

            details_fastest_checkup.forEach((checkup_details_fastest_checkup_data, index) => {
                tbl_checkup_result_fastest_response.querySelector('tbody').innerHTML += checkup_fastest_checkup_tbl_column
                    .replace('{{index_item}}', (index + 1))
                    .replace('{{checkup_results_date}}', format_date(checkup_details_fastest_checkup_data['performed']))
                    .replace('{{checkup_results_result}}', checkup_details_fastest_checkup_data['time'] + ' sec');
            });
        } else {
            tbl_checkup_result_fastest_response.querySelector('tbody').innerHTML = '';
        }

        if (details_fastest_checkup !== undefined && details_attr['failedList'] === undefined) {
            block_fastest.classList.remove('d-none');
            block_last_failures.classList.add('d-none');
        } else {
            block_fastest.classList.add('d-none');
        }
    } else {
        block_checkup_details.classList.add('d-none');
        block_no_item.classList.remove('d-none');

        block_checkup_result_url.closest('li').classList.add('d-none');
        block_checkup_result_interval.closest('li').classList.add('d-none');
        block_checkup_result_last_check_checkup_time.closest('li').classList.add('d-none');
        block_checkup_result_online_since.closest('li').classList.add('d-none');
        block_checkup_result_ssl_ssl_issuer.closest('li').classList.add('d-none');

        block_checkup_result_ssl_cipher_protocol.closest('li').classList.add('d-none');

        block_checkup_result_ssl_valid_from_to.closest('li').classList.add('d-none');
        block_checkup_result_dns_error.closest('li').classList.add('d-none');

        block_checkup_result_ip_address.closest('li').classList.add('d-none');
        block_checkup_result_domain_expires.closest('li').classList.add('d-none');
        block_checkup_result_minmax_time.closest('li').classList.add('d-none');
        block_checkup_successful_failed_checkup.closest('li').classList.add('d-none');
    }
    close_loader();
}

async function load_dashboard_list(checkup_period, checkup_name) {

    const block_dashboard_last_checkup = document.getElementById('block_dashboard_last_checkup');
    const tbl_dashboard_last_checkup_list = document.getElementById('tbl_dashboard_last_checkup_list');
    const block_slider = document.getElementById('block_slider');
    const block_slider_btn = document.getElementById('block_slider_btn');
    const graph_slider = document.getElementById('slide_container');
    const block_dashboard_last_failures = document.getElementById('block_dashboard_last_failures');
    const tbl_dashboard_last_failure_list = document.getElementById('tbl_dashboard_last_failure_list');
    const tbl_dashboard_available_from = document.getElementById('block_dashboard_available_from');

    graph_slider.innerHTML = '';

    const dashboard_details_data = await manage_api_data('checkupdetails/' + checkup_period, 'get');

    if (dashboard_details_data['data'] !== undefined) {
        const checkup_dashboard_last_checkup_column = await fetch_template('tbl_column/template_tbl_column_last_checkup.html');
        const checkup_dashboard_slider_item = await fetch_template('tbl_column/template_slider_item.html');
        const checkup_dashboard_last_failure_column = await fetch_template('tbl_column/template_tbl_column_last_failure.html');
        const dashboard_attributes = dashboard_details_data.data.attributes;

        tbl_dashboard_last_checkup_list.querySelector('tbody').innerHTML = '';
        tbl_dashboard_last_failure_list.querySelector('tbody').innerHTML = '';

        const last_checkup_list = dashboard_attributes['lastCheckups'];
        const last_checkup_failed_list = dashboard_attributes['failedList'];

        if (last_checkup_list.length > 0) {
            last_checkup_list.forEach((single_last_checkup, index) => {
                const tmp_id = single_last_checkup['checkupId'].toString();

                graph_slider.innerHTML += checkup_dashboard_slider_item.replaceAll('{{checkup_name}}', tmp_id);
                showGraphic(parseInt(tmp_id), checkup_period, 'webcheckup-graphic-' + tmp_id);

                if (last_checkup_list.length <= 1) {
                    block_slider_btn.classList.add('d-none');
                } else {
                    block_slider_btn.classList.remove('d-none');
                    block_slider.querySelector('.card-header').innerHTML = 'Website checkups ' + checkup_name;
                }

                const tmp_last_checkup_item_id = generate_unique_id();

                tbl_dashboard_last_checkup_list.querySelector('tbody').innerHTML += checkup_dashboard_last_checkup_column.replace('{{item_index}}', (index + 1))
                    .replace('{{item_id}}', tmp_last_checkup_item_id)
                    .replace('{{item_uid}}', tmp_id)
                    .replace('{{item_date}}', format_date(single_last_checkup['lastCheckupPerformed']))
                    .replace('{{item_name}}', single_last_checkup['name'])
                    .replace('{{item_link}}', single_last_checkup['link'])
                    .replaceAll('{{item_online_since}}', 'Online Since: ' + format_date(single_last_checkup['onlineSince']))
                    .replace('{{item_result}}', single_last_checkup['lastCheckupDescription'])
                    .replace('{{item_status}}', single_last_checkup['lastCheckupStatus'] !== false ? 'tick_mark' : 'close_mark')
            });
            const items = document.querySelectorAll('.carousel .carousel-item');

            items[0].classList.add('active');

        } else {
            block_dashboard_last_checkup.classList.add('d-none');
            block_slider.classList.add('d-none');
        }

        if (last_checkup_failed_list.length > 0) {
            last_checkup_failed_list.forEach((dashboard_details_last_checkup_error_data) => {
                if (dashboard_details_last_checkup_error_data.length) {
                    dashboard_details_last_checkup_error_data.forEach((dashboard_single_last_checkup, index) => {
                        tbl_dashboard_last_failure_list.querySelector('tbody').innerHTML += checkup_dashboard_last_failure_column.replace('{{item_index}}', (index + 1))
                            .replace('{{item_id}}', generate_unique_id())
                            .replace('{{item_uid}}', dashboard_single_last_checkup['checkupId'].toString())
                            .replace('{{item_times}}', dashboard_single_last_checkup['checkups'] +
                                ' time' + (parseInt(dashboard_single_last_checkup['checkups']) > 1 ? 's' : ''))
                            .replace('{{item_date}}', getTimespan(
                                dashboard_single_last_checkup['from'], dashboard_single_last_checkup['until']))
                            .replace('{{item_name}}', dashboard_single_last_checkup['checkup'])
                            .replace('{{item_result}}', dashboard_single_last_checkup['description'])
                    });
                } else {
                    tbl_dashboard_last_failure_list.querySelector('tbody').innerHTML += checkup_dashboard_last_failure_column.replace('{{item_index}}', 1)
                        .replace('{{item_id}}', generate_unique_id())
                        .replace('{{item_uid}}', dashboard_details_last_checkup_error_data['checkupId'].toString())
                        .replace('{{item_times}}', dashboard_details_last_checkup_error_data['checkups'] +
                            ' time' + (parseInt(dashboard_details_last_checkup_error_data['checkups']) > 1 ? 's' : ''))
                        .replace('{{item_date}}', getTimespan(
                            dashboard_details_last_checkup_error_data['from'], dashboard_details_last_checkup_error_data['until']))
                        .replace('{{item_name}}', dashboard_details_last_checkup_error_data['checkup'])
                        .replace('{{item_result}}', dashboard_details_last_checkup_error_data['description'])
                }
            });
            block_dashboard_last_failures.classList.remove('d-none');
        } else {
            block_dashboard_last_failures.classList.add('d-none');
        }

        if (dashboard_attributes?.firstExecutionPlanned) {
            tbl_dashboard_available_from.innerHTML = dashboard_attributes.firstExecutionPlanned;
            tbl_dashboard_available_from.classList.remove('d-none');
        }
    } else {
        generate_multiple_toast(dashboard_details_data['errors']);
        close_loader();
        document.getElementById('block_dashboard').classList.add('d-none');
    }
    close_loader();

}

function getTimespan(from, until) {
    if (from === until) {
        return format_date(from);
    } else {
        return format_date(from) + ' - ' + format_date(until);
    }
}
/* PAGE FUNCTION */

document.addEventListener('DOMContentLoaded', async function () {
    if (document.body.classList.contains('website_checkups')) {
        wc_site_url = website_checkups_code['site_url'];
        wc_api_loc = website_checkups_code['api_loc'];
        wc_nonce = website_checkups_code['nonce'];
        wc_template_loc = website_checkups_code['template_loc'];
        wc_user_data = website_checkups_code['user_data'];
        wc_page_class = website_checkups_code['page_class'];

        const post_data = {
            'post_type': 'load_form'
        }

        manage_local_data(post_data);

        AuthTokenWebsiteCheckups = wc_user_data['token'];

        page_title = document.getElementById('page_title');
        page_container = document.getElementById('page_container');

        await check_main_menu();

        if (wc_page_class === 'toplevel_page_website_checkups_menu') {

            if (!wc_user_data['token']) {
                setTimeout(async function () {
                    await registration_page();
                    close_loader();
                }, 750);
                page_title.innerText = '';

            } else {
                await setting_page();
                page_title.innerText = 'Settings';
            }
            close_loader();
        }
        if (wc_page_class === 'website-checkups_page_notifications_menu') {
            page_title.innerText = 'Notifications';
            modal_name = 'notifications';
            await notification_page();

            const info_tooltip_list = document.querySelectorAll('.info_tooltip');
            info_tooltip_list.forEach(tooltip_btn => {
                tooltip_btn.addEventListener('click', async function (e) {
                    e.preventDefault();
                    if (this.getAttribute('data-info') !== 'null') {
//                        console.log('sd');
                        show_toast('light', this.getAttribute('data-info'));
                    }
                });
            });
            close_loader();
        }
        if (wc_page_class === 'website-checkups_page_checkups_menu') {
            page_title.innerText = 'Checkups';
            modal_name = 'sitecheckups';
            await checkup_page();
            close_loader();
        }
        if (wc_page_class === 'website-checkups_page_checkup_results_menu') {
            page_title.innerText = 'Checkup Results';
            await checkup_details_page();
            close_loader();
        }
        if (wc_page_class === 'website-checkups_page_dashboard_menu') {
            page_title.innerText = 'Dashboard';
            await dashboard_page();
            close_loader();
        }
    }
});
